C     Copyright (c) 2002, NAKANO Takeo All rights reserved.
C
C     Redistribution and use in source and binary forms, with or without
C     modification, are permitted provided that the following conditions
C     are met:
C
C     Redistributions of source code must retain the above copyright
C     notice, this list of conditions and the following disclaimer.
C
C     Redistributions in binary form must reproduce the above copyright
C     notice, this list of conditions and the following disclaimer in
C     the documentation and/or other materials provided with the
C     distribution.
C
C     Neither the name of the Seikei University nor the names of its
C     contributors may be used to endorse or promote products derived
C     from this software without specific prior written permission.
C
C     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
C     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
C     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
C     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
C     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
C     BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
C     EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
C     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
C     DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
C     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
C     TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
C     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
C     SUCH DAMAGE.
C
      subroutine pfom
C
C     initialization function for fom
C
      integer NWL
      real*8 WLBEG, WLSTEP
      common /cfom1/ WLBEG, WLSTEP, NWL

      real*8 WLEND

      real*8 ledge, hedge
      common /cfom2/ ledge, hedge
      
      read(5, 2000) WLBEG
      read(5, 2000) WLEND
      read(5, 2000) WLSTEP

      NWL = int((WLEND - WLBEG)/WLSTEP) + 1
      if (NWL .LT. 0) then
         write(0,9000)
 9000    format('(pfom) no points during WL range')
      endif

      read(5, 2000) ledge
      read(5, 2000) hedge

 2000 format(f10.5)

      return
      end


      double precision function fom(X, NX)
C
C     figure of merit function, called from amoeba.
C
C     INPUTS
C       X(NX) nodes in parameter space
C       NX    dimension of X
C
C     COMMON BLOCK cfom1
C       WLBEG  ... Lower limit of sq. summation wl-range 
C       WLSTEP ... wl-step of the summation
C       NWL    ... summation iteration,
C                  (i.e., WLEND = WLBEG + WLSTEP * NWL)
C
      integer NX
      real*8 X(NX)
     
      integer NLMAX
      parameter (NLMAX = 50)

      integer NWL
      real*8 WLBEG, WLSTEP
      common /cfom1/ WLBEG, WLSTEP, NWL

      integer nl
      real*8 ri(NLMAX), d(NLMAX)
      real*8 wl, rint, tint

      real*8 tideal
      EXTERNAL tideal

      real*8 sum

      nl = NLMAX
      call fstruc(X, NX, nl, ri, d)

      sum = 0.D0
      do 10 i = 0, NWL
         WL = WLBEG + i * WLSTEP
         call multilayer(ri, d, wl, nl, rint, tint)

         sum = sum + (tint - tideal(wl))**2
 10   continue
      fom = SUM
      return
      end


      real*8 function tideal(wl)
C
C     idealized filter spectrum to be achived.
C
C     COMMON BLOCK cfom2
C       ledge ... lower  filter cutoff 
C       hedge ... higher filter cutoff
C
      real*8 wl
      real*8 ledge, hedge
      common /cfom2/ ledge, hedge

      if ((wl .le. ledge) .or. (wl .ge. hedge)) then
         tideal = 1.d0
      else
         tideal = 0.d0
      endif
      return
      end


      subroutine ospectrum(X, NX)
C
C     output spectrum
C
C     INPUT
C       X  : optimized node by amoeba.
C       NX : dimension of X
C
      integer NX
      double precision X(NX)

      integer NLMAX
      parameter (NLMAX = 50)

      integer NWL
      real*8 WLBEG, WLSTEP
      common /cfom1/ WLBEG, WLSTEP, NWL

      integer nl
      real*8 ri(NLMAX), d(NLMAX)
      real*8 wl, rint, tint

      nl = NLMAX
      call fstruc(X, NX, nl, ri, d)

      write(6, 1000)
 1000 format('# spectrum')

      do 10 i = 0, NWL
         WL = WLBEG + i * WLSTEP
         call multilayer(ri, d, wl, nl, rint, tint)

         write(6, 1010) WL, tint
 10   continue
 1010 format(2F20.9)

      return
      end
