C     Copyright (c) 2002, NAKANO Takeo All rights reserved.
C
C     Redistribution and use in source and binary forms, with or without
C     modification, are permitted provided that the following conditions
C     are met:
C
C     Redistributions of source code must retain the above copyright
C     notice, this list of conditions and the following disclaimer.
C
C     Redistributions in binary form must reproduce the above copyright
C     notice, this list of conditions and the following disclaimer in
C     the documentation and/or other materials provided with the
C     distribution.
C
C     Neither the name of the Seikei University nor the names of its
C     contributors may be used to endorse or promote products derived
C     from this software without specific prior written permission.
C
C     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
C     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
C     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
C     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
C     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
C     BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
C     EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
C     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
C     DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
C     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
C     TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
C     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
C     SUCH DAMAGE.
C
      subroutine fstruc(X, NX, NL, RI, D)
C
C     subroutine to construct filter structure
C     from the point in amoeba's paramater space
C
C     List of variables
C     INPUT
C       X(NX) nodes in parameter space
C       NX    dimension of X
C
C     OUTPUT
C       NL     :   total number of layers (initially NLMAX)
C       RI(NL) :   refractive index
C       D(NL)  :   thickness
C
C     COMMON BLOCK cfstruc
C       RIair      : Refractive index of incident side medium
C       RIglass    :                     outgoing side medium
C       RIfofs     : (RIfmax + RIfmin) / 2
C       RIfamp     : (RIfmax - RIfmin) / 2
C       WLcenter   : Wavelength of which the opt. thickness
C                    of each layer should be 1/4.
C       NLAYER     : number of layers
C
      integer NX, NL
      double precision  X(NX)
      double precision  RI(NL), D(NL)

      integer NLAYER
      double precision  RIair, RIglass, RIfofs, RIfamp, WLcenter
      common /cfstruc/RIair, RIglass, RIfofs, RIfamp, WLcenter, NLAYER
      NX = NLAYER - 2
      NL = NLAYER
C
C     data set of RI(i)
C
      RI(1) = RIair
      do 2 i= 2, NX+1
        RI(i) = RIfofs + RIfamp * sin(X(i-1))
    2 continue

      RI(NX + 2) = RIglass

C
C     D = lambda/4n
C
      do 3 i=2,NL-1
        D(i) = WLcenter/(4*RI(i))
    3 continue

      return
      end


      subroutine pfstruc(XINI, NX)
C
C     Preparation function for fstruc
C
C     List of variables
C     INPUT
C       NX    available size of xini
C
C     OUTPUT
C       XINI(NX)   : initial value of the node
C                    in amoeba's parameter space
C       NX    dimension of X
C
************  The example of an input  *************************
*      In the case of NP=5                                     *
*  5           :  Number of layers (not including both sides)  *
*  1           :  init structure (1: L 1st, -1: H 1st, 0: flat)*
*  1.0         :  Air (The refractive index of air)            *
*  1.51        :  Glass (The refractive index of Glass)        *
*  1.38        :  min value of RI of layers                    *
*  2.435       :  max value of RI of layers                    *
*  730         :  WLcenter                                     *
****************************************************************

      integer NX
      double precision  XINI(NX)

      integer NLAYER, IINIT
      double precision  RIair, RIglass, RIfofs, RIfamp, WLcenter
      common /cfstruc/RIair, RIglass, RIfofs, RIfamp, WLcenter, NLAYER

      integer NXI
      double precision RIfmin, RIfmax, PI
      parameter (PI = 3.141592654D0)

C
C     refractive index input
C
      read(5, 2000) NXI
      if (NXI .gt. NX) then
         write(0, 9010) NXI, NX
         stop
      end if
 9010 format('(fstruc) input number of layers (', I5,
     $          ') exceeds the limit:', I5)

      read(5, 2000) IINIT
      if ((IINIT .gt. 1) .or. (IINIT .lt. -1)) then
         write(0, 9020)
         stop
      end if
 9020 format('(fstruc) iinit should be either -1, 0 or 1')

      NX = NXI
      NLAYER = NXI + 2

      read(5, 2010) RIair
      read(5, 2010) RIglass

      read(5, 2010) RIfmin
      read(5, 2010) RIfmax

      RIfofs = (RIfmin + RIfmax) / 2
      RIfamp = (RIfmax - RIfmin) / 2

      do 10 i=1, NX
         if (IINIT .eq. 0) then
            XINI(i) =   0.0D0
         else if (IINIT .eq. 1) then
            XINI(i) = 0.5D0 * PI * (-1) ** i 
         else
            XINI(i) = -0.5D0 * PI * (-1) ** i
         end if
 10   continue

C
C    lambda input
C
      read(5, 2010) WLcenter

 2000 format(i10)
 2010 format(f10.5)

      return
      end
