C     Copyright (c) 2002, NAKANO Takeo All rights reserved.
C
C     Redistribution and use in source and binary forms, with or without
C     modification, are permitted provided that the following conditions
C     are met:
C
C     Redistributions of source code must retain the above copyright
C     notice, this list of conditions and the following disclaimer.
C
C     Redistributions in binary form must reproduce the above copyright
C     notice, this list of conditions and the following disclaimer in
C     the documentation and/or other materials provided with the
C     distribution.
C
C     Neither the name of the Seikei University nor the names of its
C     contributors may be used to endorse or promote products derived
C     from this software without specific prior written permission.
C
C     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
C     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
C     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
C     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
C     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
C     BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
C     EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
C     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
C     DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
C     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
C     TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
C     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
C     SUCH DAMAGE.
C
#include "fopt.h"

      PROGRAM fopt
C
C     sample program for amoeba
C
      integer NP, MP
      parameter (NP = _NMAX_, MP = NP + 1)

      double precision p(MP,NP), x(NP), y(MP)
      INTEGER iter, ndim

      double precision FTOL
      parameter (FTOL = 1.0D-6)

      double precision fom
      external fom

      double precision FACTOR
      parameter (FACTOR = 1.5D0)

      integer i, j
      double precision residual

C
C     preparation
C
      ndim = NP
      call pfstruc(x, ndim)
      call pfom

      write(0, 9000)
 9000 format('# input data successfully read.')

      do 10 i = 1, ndim
         p(1, i) = x(i)
 10   continue

C
C     create initial simplex
C
      do 20 j=2, ndim+1
         do 30 i=1, ndim
            p(j,i) = p(1,i)
 30      continue
         if (p(j,j-1) .NE. 0.0) then
            p(j,j-1) = p(j,j-1)*FACTOR
         else
            p(j,j-1) = p(j,j-1)+FACTOR
         endif
 20   continue

      do 40 j=1, ndim+1
         do 50 i=1,ndim
            x(i)=p(j,i)
 50      continue
         y(j)=fom(x,ndim)
 40   continue

C
C     optimization
C
      call amoeba(p,y,MP,NP,ndim,FTOL,fom,iter)

C
C     output
C
      write(0, 9010) iter
 9010 format('# number of iterations:', i10)

      do 60 i = 1, ndim
         x(i) = p(1,i)
 60   continue

      residual = fom(x, ndim)
      write(0, 9020) residual
 9020 format('# residual error      :', f20.9)

      call ofstruc(x, ndim)
      call ospectrum(x, ndim)
      stop
      end
